<?php
class Router
{
    public Request $request;
    public Response $response;
    protected array $routes = [];


    public function __construct(Request $request, Response $response)
    {
        $this->request = $request;
        $this->response = $response;
    }

    public function get($path, $callback)
    {
        $this->routes['get'][$path] = $callback;
    }

    public function post($path, $callback)
    {
        $this->routes['post'][$path] = $callback;
    }



    public function resolve()
    {
        global $currentPage;
        $path = $this->request->getPath();
        $path = rtrim($path,'/');
        $method = $this->request->method();
        

        foreach ($this->routes[$method] as $routePath => $callback) {
            $routePath = preg_replace('/\{(.*?)\}/', '(.*?)', $routePath);
            if (preg_match("#^$routePath$#", $path, $matches)) {
                array_shift($matches);
                if (is_string($callback)) {
                    return $this->view($callback, $matches);
                }
                if (is_array($callback)) {
                    require "../app/controllers/" . $callback[0] . ".php";
                    $controller = new $callback[0]();
                   // return call_user_func_array([$controller, $callback[1]], [$this->request]);
                    return call_user_func_array([$controller, $callback[1]], [$this->request, ...$matches]);
                }
                return call_user_func_array($callback, $matches);
            }
        }
        $this->response->setStatusCode(404);
        return "Not Found";
    }


    public function view($view, $params = [])
    {
        $layout = $this->layout($params = []);
        $content = $this->onlyView($view, $params);
      //  return str_replace('{{content}}', $content, $layout);
        return preg_replace('/{{\s*content\s*}}/', $content, $layout);
    }

    protected function layout($params = [])
    {
        ob_start();
        include_once "../app/views/layout/main.php";
        return ob_get_clean();
    }

    protected function onlyView($view, $params)
    {
        ob_start();
        include_once "../app/views/$view.php";
        return ob_get_clean();
    }

    public function isActiveLink($url) {
        if ($this->request->getPath() == $url) {
            return "active";
        } else {
            return "";
        }
    }



}
